/*
Colette Lee
MATT259A Data Visualization - George Legrady

Checkouts of Items related to Astrology

Usage: Click the checkboxes on the left to toggle each item on/off
Click 'Titles' on the top left to hide all titles
Use up/down arrow keys to scroll titles
Click Activate All / Deactivate All buttons on the top right
*/


import controlP5.*;

import peasy.*;

PeasyCam cam;
ControlP5 cp5;
CheckBox checkbox;


int checkHeight = 15;

int numYears = 15;
int size = 300;
int zDist = 50;
int startYear = 2006;
int curveSize = 350;
float startAngle = -PI/2;

Table table;
int numRows, numColumns;

ArrayList<Integer> bibNumbers;
int highlighted  = 0;

HashMap<Integer, ArrayList<Checkout>> bookMap;

String[] months = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sept", "Oct", "Nov", "Dec"};

//polynomail regression coefficients computed by https://arachnoid.com/polysolve/
float[] coefficients = {
8.2805696644915122E+2, 
-5.7777354911398061E+2, 
7.2808650801137867E+2, 
-3.2267661146756541E+2,
 6.5701565603183184E+1,
-6.8094595535756355E+0,
 3.4982413442129745E-1,
-7.0719077709594097E-3};


void setup(){
  //size(1300,800,P3D);
  fullScreen(P3D);
  cam = new PeasyCam(this, 200);
  
  smooth(8);
  
  table = loadTable("astrologyall.csv","header");
    
  bookMap = new HashMap<Integer, ArrayList<Checkout>>();
  
  bibNumbers = new ArrayList<Integer>();
  
  numRows = table.getRowCount();    
  
  for(int i=0; i<this.numRows;i++){
    int itemNumber = table.getInt(i,0);
    int bibNumber  = table.getInt(i,1);
    String coutDate = table.getString(i,2);
    String cinDate = table.getString(i,3);
    String coutTime = table.getString(i,4);
    String cinTime = table.getString(i,5);
    String title = table.getString(i,6);
    
    if(dateParser(coutDate)<0)
    println(i, coutDate,  dateParser(coutDate), title);
    
    Checkout newCheck = new Checkout(coutDate,coutTime,cinDate,cinTime,title,itemNumber, bibNumber);
    
    if(!bibNumbers.contains(bibNumber)){
      bibNumbers.add(bibNumber);
    }
    
    if(!bookMap.containsKey(bibNumber)){
      bookMap.put(bibNumber,new ArrayList<Checkout>());   
    } 
    bookMap.get(bibNumber).add(newCheck);  
    
  }
  //println(bibNumbers.size());
  
  setupControls();

}

void draw(){
  background(0);
  noFill();
  float t = startAngle;
  
  translate(0,0,zDist*numYears*3);
  
  
  // month/year labels
  textSize(30);
  for(int i=0; i<numYears*12; i++){
    float index = map(i,0,numYears*12,0,14);
    float m =  getPoly(index);
    float x = m*size*cos(t);
    float y = m*size*sin(t);
    float z = -zDist*t;

    push();
    translate(x,y,z);
    scale(0.4);
    if(i%12<4){
      textAlign(LEFT,BOTTOM);
    } else if (i%12>=4 && i%12<7){
      textAlign(LEFT,TOP);
    } else if (i%12>=7 && i%12<10){
      textAlign(RIGHT,TOP);
    } else if (i%12>=10 && i%12<=12){
      textAlign(RIGHT,BOTTOM);
    }
    text(months[i%12],0,0);
    if(i%12==0){
      translate(0,-75,0);
      scale(2);
      textAlign(CENTER);
      text((i/12 + startYear),0,0);
    }
    pop();
    
    t+= 2*PI/12;
  }


  
  
  //displaying checkouts as Bezier curves
  for(int j=0; j<bibNumbers.size();j++){
    if(checkbox.getState(j)){
    ArrayList<Checkout> cList = bookMap.get(bibNumbers.get(j));
    for(int i=0;i<cList.size();i++){
      Checkout c = cList.get(i);
      
      int coutDays = dateParser(c.coutDate);
      int cinDays = dateParser(c.cinDate);
     float a = startAngle + map(coutDays,0,365*numYears,0,2*PI*numYears);
     float b = startAngle + map(cinDays,0,365*numYears,0,2*PI*numYears);
     
     int minutes = timeParser(c.coutTime);
     float h1 = map(minutes,10*20,20*60, 0,curveSize);
     
     int minutesIn = timeParser(c.cinTime);
     float h2 = map(minutesIn,10*20,20*60, 0,curveSize);
     
      
     
     float indexA = map(coutDays,0,365*numYears,0,14);
     float indexB = map(cinDays,0,365*numYears,0,14);
      
      float mA = getPoly(indexA);
      float mB = getPoly(indexB);
      

     float x1 = mA*size*cos(a);
     float y1 = mA*size*sin(a);
     float z1 = -zDist*a;
     
     float x2 = mB*size*cos(b);
     float y2 = mB*size*sin(b);
     float z2 = -zDist*b;
     
     
     //display titles when hovering
     float  mappedX = screenX(x1,y1,z1);
     float  mappedY = screenY(x1,y1,z1);
     float distance = dist(mouseX,mouseY,mappedX,mappedY);
     
     float  mappedX2 = screenX(x2,y2,z2);
     float  mappedY2 = screenY(x2,y2,z2);
     float distance2 = dist(mouseX,mouseY,mappedX2,mappedY2);
     if(distance<3 ||  distance2<4){
        pushMatrix();
        if(distance<3){
          translate(x1,y1,z1);
        } else {
          translate(x2,y2,z2);
        }
          
        textAlign(LEFT,BOTTOM);
        scale(0.4);
        text(c.title,0,0);
        popMatrix();
        highlighted = c.bibNumber;
      } else {
        highlighted = 0;
      }
     
     
     stroke(getColor(c.bibNumber));
     if(highlighted == c.bibNumber){
       strokeWeight(4);
     } else  {
       strokeWeight(1);
     }
     
     if(coutDays!=-13139)
     bezier(x1,y1,z1,x1,y1,z1-h1,x2,y2,z2+h2,x2,y2,z2);
     
    }
    }
  }
  gui();
  
}
